package com.ibm.tools.attach.spi;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2009, 2010  All Rights Reserved
 */

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import com.ibm.tools.attach.AttachNotSupportedException;
import com.ibm.tools.attach.AttachPermission;
import com.ibm.tools.attach.VirtualMachine;
import com.ibm.tools.attach.VirtualMachineDescriptor;
import com.ibm.tools.attach.javaSE.Permissions;

/**
 * This class is analogous to and intended to be compatible with
 * com.sun.tools.attach.spi.AttachProvider.
 *
 * @see "http://w3.hursley.ibm.com/java/docs/java7/jdk/api/attach/spec/com/sun/tools/attach/spi/AttachProvider.html"
 */
public abstract class AttachProvider {

	private static ArrayList<AttachProvider> providers;

	/**
	 *
	 * @throws SecurityException
	 *             Not used at present
	 * @note Public API, signature compatible with
	 *       com.sun.tools.attach.spi.AttachProvider.
	 */
	protected AttachProvider() {
		final SecurityManager securityManager = System.getSecurityManager();
		if (securityManager != null) {
			securityManager.checkPermission(Permissions.CREATE_ATTACH_PROVIDER);
		}

		return;
	}

	/**
	 * @return name of provider
	 * @note Public API, signature compatible with
	 *       com.sun.tools.attach.spi.AttachProvider.
	 */
	public abstract String name();

	/**
	 * @return protocol type
	 */
	public abstract String type();

	/**
	 * @param id
	 *            of VM to which to attach
	 * @return attached VM
	 * @throws AttachNotSupportedException
	 *             thrown if the target does not respond
	 * @throws IOException
	 *             on access to control files
	 * @note Public API, signature compatible with
	 *       com.sun.tools.attach.spi.AttachProvider.
	 */
	public abstract VirtualMachine attachVirtualMachine(String id)
			throws AttachNotSupportedException, IOException;

	/**
	 * @param vmd
	 *            descriptor of target VM
	 * @return target VM
	 * @throws AttachNotSupportedException
	 *             thrown if the target does not respond
	 * @throws IOException
	 *             on access to control files
	 * @note Public API, signature compatible with
	 *       com.sun.tools.attach.spi.AttachProvider.
	 */
	public abstract VirtualMachine attachVirtualMachine(
			VirtualMachineDescriptor vmd) throws AttachNotSupportedException,
			IOException;

	/**
	 * @return list of available target VMs
	 * @note Public API, signature compatible with
	 *       com.sun.tools.attach.spi.AttachProvider.
	 */
	public abstract List<VirtualMachineDescriptor> listVirtualMachines();

	/**
	 * @return list of providers
	 * @note Public API, signature compatible with
	 *       com.sun.tools.attach.spi.AttachProvider.
	 */
	public static synchronized List<AttachProvider> providers() {

		if (providers != null) {
			return providers;
		}

		providers = new ArrayList<AttachProvider>();

		String providersProperty = System
				.getProperty("com.ibm.attach.providers"); //$NON-NLS-1$
		if ((null == providersProperty) || (0 == providersProperty.length())) {
			/* If we don't have an explicit providers */
			providers.add(new com.ibm.tools.attach.javaSE.AttachProviderImpl());
			return providers;
		}

		/* Otherwise the property must be a list of class names, resolve them */
		StringTokenizer providerClasses = new StringTokenizer(
				providersProperty, ":;"); //$NON-NLS-1$
		while (providerClasses.hasMoreTokens()) {
			Exception exception = null;
			String providerClassName = providerClasses.nextToken();

			try {
				/* load the class and create a new instance */
				Class co = Class.forName(providerClassName);
				providers.add((AttachProvider) co.newInstance());
			} catch (ClassNotFoundException e) {
				exception = e;
			} catch (IllegalAccessException e) {
				exception = e;
			} catch (InstantiationException e) {
				exception = e;
			}

			/*
			 * If an exception occurred, clear the providers (allow retry) and
			 * throw IllegalArgumentException
			 */
			if (exception != null) {
				providers = null;

				// K0540 = cannot load AttachProvider {0} reason: {1}
				String msg = com.ibm.oti.util.Msg.getString("K0540", providerClassName, exception.getClass().getName()); //$NON-NLS-1$
				throw new IllegalArgumentException(msg);
			}
		}
		return providers;
	}
}
